@@ -3,6 +3,7 @@ use serde::{Deserialize, Serialize};
 use std::fmt;
 
 mod invite_link;
+mod join_request;
 mod location;
 mod member;
 mod permissions;
@@ -10,6 +11,7 @@ mod photo;
 
 pub use self::{
     invite_link::ChatInviteLink,
+    join_request::ChatJoinRequest,
     location::ChatLocation,
     member::{ChatMember, ChatMemberAdministrator, ChatMemberKicked, ChatMemberRestricted, ChatMemberUpdated},
     permissions::ChatPermissions,
@@ -83,6 +85,10 @@ pub struct ChannelChat {
     ///
     /// Returned only in getChat
     pub linked_chat_id: Option<Integer>,
+    /// True, if messages from the chat can't be forwarded to other chats
+    ///
+    /// Returned only in getChat.
+    pub has_protected_content: Option<bool>,
 }
 
 /// Group chat
@@ -108,6 +114,10 @@ pub struct GroupChat {
     ///
     /// Returned only in getChat
     pub permissions: Option<ChatPermissions>,
+    /// True, if messages from the chat can't be forwarded to other chats
+    ///
+    /// Returned only in getChat.
+    pub has_protected_content: Option<bool>,
 }
 
 /// Private chat
@@ -133,6 +143,12 @@ pub struct PrivateChat {
     ///
     /// Returned only in getChat
     pub pinned_message: Option<Box<Message>>,
+    /// True, if privacy settings of the other party
+    /// in the private chat allows to use tg://user?id=<user_id>
+    /// links only in chats with the user.
+    ///
+    /// Returned only in getChat
+    pub has_private_forwards: Option<bool>,
 }
 
 /// Supergroup chat
@@ -184,6 +200,10 @@ pub struct SupergroupChat {
     ///
     /// Returned only in getChat
     pub location: Option<ChatLocation>,
+    /// True, if messages from the chat can't be forwarded to other chats
+    ///
+    /// Returned only in getChat.
+    pub has_protected_content: Option<bool>,
 }
 
 /// Chat ID or username
@@ -227,6 +247,8 @@ impl From<Integer> for ChatId {
 #[derive(Clone, Copy, Debug, Hash, PartialEq, PartialOrd, Serialize)]
 #[serde(rename_all = "snake_case")]
 pub enum ChatAction {
+    /// For stickers
+    ChooseSticker,
     /// For location data
     FindLocation,
     /// For videos
@@ -279,7 +301,8 @@ mod tests {
                 },
                 "text": "test"
             },
-            "linked_chat_id": 2
+            "linked_chat_id": 2,
+            "has_protected_content": true
         }))
         .unwrap();
         assert_eq!(chat.get_id(), 1);
@@ -297,6 +320,7 @@ mod tests {
             assert_eq!(chat.invite_link.unwrap(), "channelinvitelink");
             assert!(chat.pinned_message.is_some());
             assert_eq!(chat.linked_chat_id.unwrap(), 2);
+            assert!(chat.has_protected_content.unwrap());
         } else {
             panic!("Unexpected chat: {:?}", chat);
         }
@@ -353,7 +377,8 @@ mod tests {
                 },
                 "text": "test"
             },
-            "permissions": {"can_send_messages": true}
+            "permissions": {"can_send_messages": true},
+            "has_protected_content": true
         }))
         .unwrap();
         assert_eq!(chat.get_id(), 1);
@@ -370,6 +395,7 @@ mod tests {
             let permissions = chat.permissions.unwrap();
             assert!(permissions.can_send_messages.unwrap());
             assert!(chat.pinned_message.is_some());
+            assert!(chat.has_protected_content.unwrap());
         } else {
             panic!("Unexpected chat: {:?}", chat);
         }
@@ -426,7 +452,8 @@ mod tests {
                     "first_name": "user"
                 },
                 "text": "test"
-            }
+            },
+            "has_private_forwards": true
         }))
         .unwrap();
         assert_eq!(chat.get_id(), 1);
@@ -443,6 +470,7 @@ mod tests {
             assert_eq!(photo.big_file_unique_id, "bigfileuniqueid");
             assert_eq!(chat.bio.unwrap(), "testbio");
             assert_eq!(chat.pinned_message.unwrap().id, 1);
+            assert!(chat.has_private_forwards.unwrap());
         } else {
             panic!("Unexpected chat: {:?}", chat)
         }
@@ -513,7 +541,8 @@ mod tests {
                     "latitude": 1
                 },
                 "address": "test location"
-            }
+            },
+            "has_protected_content": true
         }))
         .unwrap();
         assert_eq!(chat.get_id(), 1);
@@ -537,6 +566,7 @@ mod tests {
             assert!(permissions.can_send_messages.unwrap());
             assert_eq!(chat.linked_chat_id.unwrap(), 2);
             assert_eq!(chat.location.unwrap().address, "test location");
+            assert!(chat.has_protected_content.unwrap());
         } else {
             panic!("Unexpected chat: {:?}", chat)
         }
@@ -608,6 +638,10 @@ mod tests {
 
     #[test]
     fn chat_action() {
+        assert_eq!(
+            serde_json::to_string(&ChatAction::ChooseSticker).unwrap(),
+            r#""choose_sticker""#
+        );
         assert_eq!(
             serde_json::to_string(&ChatAction::FindLocation).unwrap(),
             r#""find_location""#
